/*
 * Copyright (C) 2001-2015 Klaralvdalens Datakonsult AB.  All rights reserved.
 *
 * This file is part of the KD Chart library.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TERNARYCONSTANTS_H
#define TERNARYCONSTANTS_H

#include <QPointF>

extern const qreal Sqrt3;
extern const qreal TriangleWidth;
extern const qreal TriangleHeight;
extern const QPointF TriangleTop;
extern const QPointF TriangleBottomLeft;
extern const QPointF TriangleBottomRight;
extern const QPointF AxisVector_C_A;
extern const QPointF Norm_C_A;
extern const QPointF AxisVector_B_A;
extern const QPointF Norm_B_A;
extern const QPointF AxisVector_B_C;
extern const QPointF Norm_B_C;
extern const qreal RelMarkerLength;
extern const QPointF FullMarkerDistanceBC;
extern const QPointF FullMarkerDistanceAC;
extern const QPointF FullMarkerDistanceBA;

#endif
